@@ -217,20 +217,18 @@ def recursive_property(strategy: "SearchStrategy", name: str, default: object) -
 
 
 class SearchStrategy(Generic[Ex]):
-    """A SearchStrategy is an object that knows how to explore data of a given
-    type.
+    """A ``SearchStrategy`` tells Hypothesis how to generate that kind of input.
 
-    Except where noted otherwise, methods on this class are not part of
-    the public API and their behaviour may change significantly between
-    minor version releases. They will generally be stable between patch
-    releases.
+    This class is only part of the public API for use in type annotations, so that
+    you can write e.g. ``-> SearchStrategy[Foo]`` for your function which returns
+    ``builds(Foo, ...)``.  Do not inherit from or directly instantiate this class.
     """
 
     validate_called: bool = False
     __label: Union[int, UniqueIdentifier, None] = None
     __module__: str = "hypothesis.strategies"
 
-    def available(self, data: ConjectureData) -> bool:
+    def _available(self, data: ConjectureData) -> bool:
         """Returns whether this strategy can *currently* draw any
         values. This typically useful for stateful testing where ``Bundle``
         grows over time a list of value to choose from.
@@ -287,15 +285,12 @@ class SearchStrategy(Generic[Ex]):
     def calc_has_reusable_values(self, recur: RecurT) -> bool:
         return False
 
-    def example(self) -> Ex:
-        """Provide an example of the sort of value that this strategy
-        generates. This is biased to be slightly simpler than is typical for
-        values from this strategy, for clarity purposes.
+    def example(self) -> Ex:  # FIXME
+        """Provide an example of the sort of value that this strategy generates.
 
-        This method shouldn't be taken too seriously. It's here for interactive
-        exploration of the API, not for any sort of real testing.
-
-        This method is part of the public API.
+        This method is designed for use in a REPL, and will raise an error if
+        called from inside |@given| or a strategy definition.  For serious use,
+        see |@composite| or |st.data|.
         """
         if getattr(sys, "ps1", None) is None:  # pragma: no branch
             # The other branch *is* covered in cover/test_examples.py; but as that
@@ -304,7 +299,7 @@ class SearchStrategy(Generic[Ex]):
                 "The `.example()` method is good for exploring strategies, but should "
                 "only be used interactively.  We recommend using `@given` for tests - "
                 "it performs better, saves and replays failures to avoid flakiness, "
-                "and reports minimal examples. (strategy: %r)" % (self,),
+                f"and reports minimal examples. (strategy: {self!r})",
                 NonInteractiveExampleWarning,
                 stacklevel=2,
             )
@@ -361,10 +356,9 @@ class SearchStrategy(Generic[Ex]):
         return self.__examples.pop()
 
     def map(self, pack: Callable[[Ex], T]) -> "SearchStrategy[T]":
-        """Returns a new strategy that generates values by generating a value
-        from this strategy and then calling pack() on the result, giving that.
-
-        This method is part of the public API.
+        """Returns a new strategy which generates a value from this one, and
+        then returns ``pack(value)``.  For example, ``integers().map(str)``
+        could generate ``str(5)`` == ``"5"``.
         """
         if is_identity_function(pack):
             return self  # type: ignore  # Mypy has no way to know that `Ex == T`
@@ -372,16 +366,25 @@ class SearchStrategy(Generic[Ex]):
 
     def flatmap(
         self, expand: Callable[[Ex], "SearchStrategy[T]"]
-    ) -> "SearchStrategy[T]":
-        """Returns a new strategy that generates values by generating a value
-        from this strategy, say x, then generating a value from
-        strategy(expand(x))
+    ) -> "SearchStrategy[T]":  # FIXME
+        """Old syntax for a special case of |@composite|:
 
-        This method is part of the public API.
+        .. code-block:: python
+
+            @st.composite
+            def flatmap_like(draw, base_strategy, expand):
+                value = draw(base_strategy)
+                new_strategy = expand(value)
+                return draw(new_strategy)
+
+        We find that the greater readability of |@composite| usually outweighs
+        the verbosity, with a few exceptions for simple cases or recipes like
+        ``from_type(type).flatmap(from_type)`` ("pick a type, get a strategy for
+        any instance of that type, and then generate one of those").
         """
         from hypothesis.strategies._internal.flatmapped import FlatMapStrategy
 
-        return FlatMapStrategy(expand=expand, strategy=self)
+        return FlatMapStrategy(self, expand=expand)
 
     # Note that we previously had condition extracted to a type alias as
     # PredicateT. However, that was only useful when not specifying a relationship
@@ -393,11 +396,21 @@ class SearchStrategy(Generic[Ex]):
     # reference the local TypeVar context.
     def filter(self, condition: Callable[[Ex], Any]) -> "SearchStrategy[Ex]":
         """Returns a new strategy that generates values from this strategy
-        which satisfy the provided condition. Note that if the condition is too
-        hard to satisfy this might result in your tests failing with
-        Unsatisfiable.
+        which satisfy the provided condition.
 
-        This method is part of the public API.
+        Note that if the condition is too hard to satisfy this might result
+        in your tests failing with an Unsatisfiable exception.
+        A basic version of the filtering logic would look something like:
+
+        .. code-block:: python
+
+            @st.composite
+            def filter_like(draw, strategy, condition):
+                for _ in range(3):
+                    value = draw(strategy)
+                    if condition(value):
+                        return value
+                assume(False)
         """
         return FilteredStrategy(conditions=(condition,), strategy=self)
 
@@ -524,7 +537,9 @@ class SampledFromStrategy(SearchStrategy[Ex]):
     def __init__(
         self,
         elements: Sequence[Ex],
-        repr_: Optional[str] = None,
+        *,
+        force_repr: Optional[str] = None,
+        force_repr_braces: Optional[tuple[str, str]] = None,
         transformations: tuple[
             tuple[Literal["filter", "map"], Callable[[Ex], Any]],
             ...,
@@ -533,13 +548,17 @@ class SampledFromStrategy(SearchStrategy[Ex]):
         super().__init__()
         self.elements = cu.check_sample(elements, "sampled_from")
         assert self.elements
-        self.repr_ = repr_
+        self.force_repr = force_repr
+        self.force_repr_braces = force_repr_braces
         self._transformations = transformations
 
+        self._cached_repr: Optional[str] = None
+
     def map(self, pack: Callable[[Ex], T]) -> SearchStrategy[T]:
         s = type(self)(
             self.elements,
-            repr_=self.repr_,
+            force_repr=self.force_repr,
+            force_repr_braces=self.force_repr_braces,
             transformations=(*self._transformations, ("map", pack)),
         )
         # guaranteed by the ("map", pack) transformation
@@ -548,30 +567,79 @@ class SampledFromStrategy(SearchStrategy[Ex]):
     def filter(self, condition: Callable[[Ex], Any]) -> SearchStrategy[Ex]:
         return type(self)(
             self.elements,
-            repr_=self.repr_,
+            force_repr=self.force_repr,
+            force_repr_braces=self.force_repr_braces,
             transformations=(*self._transformations, ("filter", condition)),
         )
 
-    def __repr__(self) -> str:
-        return (
-            self.repr_
-            or "sampled_from(["
-            + ", ".join(map(get_pretty_function_description, self.elements))
-            + "])"
-        ) + "".join(
-            f".{name}({get_pretty_function_description(f)})"
-            for name, f in self._transformations
-        )
+    def __repr__(self):
+        if self._cached_repr is None:
+            rep = get_pretty_function_description
+            elements_s = (
+                ", ".join(rep(v) for v in self.elements[:512]) + ", ..."
+                if len(self.elements) > 512
+                else ", ".join(rep(v) for v in self.elements)
+            )
+            braces = self.force_repr_braces or ("(", ")")
+            instance_s = (
+                self.force_repr or f"sampled_from({braces[0]}{elements_s}{braces[1]})"
+            )
+            transforms_s = "".join(
+                f".{name}({get_pretty_function_description(f)})"
+                for name, f in self._transformations
+            )
+            repr_s = instance_s + transforms_s
+            self._cached_repr = repr_s
+        return self._cached_repr
 
     def calc_label(self) -> int:
-        return combine_labels(
-            self.class_label,
-            *(
-                (calc_label_from_hash(self.elements),)
-                if is_hashable(self.elements)
-                else ()
-            ),
-        )
+        # strategy.label is effectively an under-approximation of structural
+        # equality (i.e., some strategies may have the same label when they are not
+        # structurally identical). More importantly for calculating the
+        # SampledFromStrategy label, we might have hash(s1) != hash(s2) even
+        # when s1 and s2 are structurally identical. For instance:
+        #
+        #   s1 = st.sampled_from([st.none()])
+        #   s2 = st.sampled_from([st.none()])
+        #   assert hash(s1) != hash(s2)
+        #
+        # (see also test cases in test_labels.py).
+        #
+        # We therefore use the labels of any component strategies when calculating
+        # our label, and only use the hash if it is not a strategy.
+        #
+        # That's the ideal, anyway. In reality the logic is more complicated than
+        # necessary in order to be efficient in the presence of (very) large sequences:
+        # * add an unabashed special case for range, to avoid iteration over an
+        #   enormous range when we know it is entirely integers.
+        # * if there is at least one strategy in self.elements, use strategy label,
+        #   and the element hash otherwise.
+        # * if there are no strategies in self.elements, take the hash of the
+        #   entire sequence. This prevents worst-case performance of hashing each
+        #   element when a hash of the entire sequence would have sufficed.
+        #
+        # The worst case performance of this scheme is
+        # itertools.chain(range(2**100), [st.none()]), where it degrades to
+        # hashing every int in the range.
+
+        if isinstance(self.elements, range) or (
+            is_hashable(self.elements)
+            and not any(isinstance(e, SearchStrategy) for e in self.elements)
+        ):
+            return combine_labels(self.class_label, calc_label_from_hash(self.elements))
+
+        labels = [self.class_label]
+        for element in self.elements:
+            if not is_hashable(element):
+                continue
+
+            labels.append(
+                element.label
+                if isinstance(element, SearchStrategy)
+                else calc_label_from_hash(element)
+            )
+
+        return combine_labels(*labels)
 
     def calc_has_reusable_values(self, recur: RecurT) -> bool:
         # Because our custom .map/.filter implementations skip the normal
@@ -741,13 +809,13 @@ class OneOfStrategy(SearchStrategy[Ex]):
     def do_draw(self, data: ConjectureData) -> Ex:
         strategy = data.draw(
             SampledFromStrategy(self.element_strategies).filter(
-                lambda s: s.available(data)
+                lambda s: s._available(data)
             )
         )
         return data.draw(strategy)
 
     def __repr__(self) -> str:
-        return "one_of(%s)" % ", ".join(map(repr, self.original_strategies))
+        return "one_of({})".format(", ".join(map(repr, self.original_strategies)))
 
     def do_validate(self) -> None:
         for e in self.element_strategies:
@@ -773,8 +841,8 @@ class OneOfStrategy(SearchStrategy[Ex]):
 
 @overload
 def one_of(
-    __args: Sequence[SearchStrategy[Any]],
-) -> SearchStrategy[Any]:  # pragma: no cover
+    __args: Sequence[SearchStrategy[Ex]],
+) -> SearchStrategy[Ex]:  # pragma: no cover
     ...
 
 
@@ -1119,6 +1187,6 @@ def check_strategy(arg: object, name: str = "") -> None:
         if name:
             name += "="
         raise InvalidArgument(
-            "Expected a SearchStrategy%s but got %s%r (type=%s)"
-            % (hint, name, arg, type(arg).__name__)
+            f"Expected a SearchStrategy{hint} but got {name}{arg!r} "
+            f"(type={type(arg).__name__})"
         )
